# SUSE-specific operations
#
# Copyright 2008 Novell, Inc.
# Author: Wei Kong <wkong@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from cStringIO import StringIO
import glob
import gzip
import os
import re
import shutil
import stat
import tempfile
import urlgrabber.grabber as grabber
import commands,sys,getopt,tempfile,shutil

from VMLinux import VMLinux
from exceptions import *
from log import log
import caps
import mount
import msg
import util
import vmdisks
import vminstall
import random
from VMSUSE import die_zero

autoyast_vdev = vmdisks.get_possible_vdevs(False,False)[-1]

def tftp_success(statusoutput):
    errorre=re.compile("Error*")
    timeout=re.compile("Transfer timed out.")
    if errorre.match(statusoutput[1]) or timeout.match(statusoutput[1]):
        return False
    else:
        return True

def getServAddr(options):
    #Can't use dhclient, which will make network unstable
    #FIXME 1: is there any better method to get the dhcp server IP 
    #which doesn't effect network status
    #This method depend on udhcp-0.9.8-2
    #udhcpc_command="/usr/bin/udhcpc"
    udhcpc_command="/sbin/udhcpc"
    udhcpc_script="/usr/share/udhcpc/outputpy.udhcp.sh"
    nics = options.nics
    if (nics is not None and nics[0].bridge is not None):
        bridge = nics[0].bridge
    else:
        rfd = os.popen("/sbin/brctl show")
        data = rfd.read()
        status = rfd.close()
        lines = data.splitlines()[1:]
        line1 = lines[0]
        words = line1.split()
        bridge = words[0]
        
    log.debug("getServAddr: bridge: %s " % (bridge))
    udhcp_result=commands.getstatusoutput(udhcpc_command +
                               " -i " + bridge +
                               " -n -q -s " + udhcpc_script)

    if (udhcp_result[0] != 0):
        raise PXEError(PXEError.E_NO_SERVER)
    udhcplines=udhcp_result[1].split('\n')
    dhcpinfo={}
    for line in udhcplines:
        s = line.strip()
        if (s.find('=') != -1):
            f=s.split('=')
            dhcpinfo[f[0]]=f[1]

    sa=dhcpinfo['siaddr']
    log.debug('Found PXE Server address (siaddr) = %s' % sa)
    return sa

class VMPXE(VMLinux):
    global servaddr
    global kernel_list
    global initrd_list
    global pxe_label

    def __init__(self):
        self.__initialized = True

    pxe_label=''
    kernel_list={}
    initrd_list={}

    def _getMediaRoots(self, root, _open=open):
        """Network install sources may be composed of multiple CDs.  YaST lists
        their location and ordering in ./yast/instorder.  Potentially any of
        these CDs may contain a PV kernel.
        """
        roots = ['.']
        try:
            fd = _open('%s/yast/instorder' % root)
            try:
                roots = fd.readlines()
                roots = map(lambda s: s.strip(), roots)
            finally:
                fd.close()
        finally:
            log.debug("Media roots: %s" % str(roots))
            return roots

    def getInstallImagesFromPath(self, url, smboptions, progress=lambda f:None, supports_32nonpae=True):
        """Retrieves a Xen-enabled SUSE kernel and initrd from the URL, and
        copies them to temporary files.  Returns the (kernel, initrd) tuple of
        filenames."""
        ftp = url.startswith('ftp://')
        http = url.startswith('http://')
        nfs = url.startswith('nfs://')
        smb = url.startswith('smb://')
        if nfs or smb:
            import urlparse
            # NOTE:  Python's urlparse module does not recognize nfs or smb,
            #        but the code handles it okay.  Add support:
            urlparse.uses_netloc.extend(['nfs', 'smb'])
            (_, host, path, _, _, _) = urlparse.urlparse(url)
            host = host.strip(':')
            if nfs:
                mnt = mount.TemporaryNFSMount(host, path)
            else:
                mnt = mount.TemporarySMBMount(host, path, smboptions)
            base = mnt.mount()
            urlopen = lambda path: open(path)
        elif ftp or http:
            base = url
            urlopen = lambda path: grabber.urlopen(path)
        else:
            # This should never happen (unless VMDefaults are out of sync?)
            raise InstSrcError(err=InstSrcError.E_UNSUPPORTED, details=url)
        progress(0.1)

        arches = caps.compatible_arches(supports_32nonpae)
        try:
            for mediaroot in self._getMediaRoots(base, urlopen):
                #one of mediaroot is '.' so it is fine either for single-media sources
                for a in arches:
                    kfn = '/%s/boot/%s/vmlinuz-xen' % (mediaroot, a[0])
                    ifn = '/%s/boot/%s/initrd-xen' % (mediaroot, a[0])
                    log.debug("Checking for %s..." % (base+kfn))
                    try:
                        kfd = urlopen(base + kfn)
                    except:
                        continue
                    self.kfn = util.copy_to_temp(kfd, prefix='kernel.', progress=lambda f:progress(0.1+f*0.45))
                    kfd.close()
                    die_zero(self.kfn)
                    try:
                        ifd = urlopen(base + ifn)
                    except:
                        continue
                    self.ifn = util.copy_to_temp(ifd, prefix='install-initrd.', progress=lambda f:progress(0.55+f*0.45))
                    ifd.close()
                    die_zero(self.kfn)
                    log.debug("Unpacked kernel/initrd found: %s %s" % (kfn, ifn))
                    return (self.kfn, self.ifn)

        except grabber.URLGrabError, e:
            raise InstSrcError(urlgrabber=e.errno, details='%s\n%s' % (url, e.strerror))
        except Error, e:
            raise
        except Exception, e:
            raise InstSrcError(err=InstSrcError.E_BAD, details='%s\n%s' % (url, str(e)))

        log.debug("No unpacked kernel/initrd found")
        progress(0.2)
        return (None, None)

    def getPXEconfigFromPXEserver(self, options):
        try:
            servaddr=getServAddr(options)
        except Exception, e:
            raise e
        os.chdir(vminstall.paths.imagedir)
        commandstr="tftp "+servaddr+" -c get pxelinux.cfg/default"
        #commandstr="atftp -g -r pxelinux.cfg/default -l default 147.2.207.153" #for test
        cfgfilename="default"
        getpxeres=commands.getstatusoutput(commandstr)
        if not tftp_success(getpxeres) or not os.path.exists(cfgfilename):
            raise PXEError(PXEError.E_NO_CONFIG)
        cfgfile=open(cfgfilename)
        cfglines=cfgfile.readlines()
        # parse configure
        defaultrc=re.compile("default*", re.IGNORECASE)
        labelre=re.compile("label*", re.IGNORECASE)
        kernelre=re.compile("kernel*", re.IGNORECASE)
        appendre=re.compile("append*", re.IGNORECASE)
        initrdre=re.compile("initrd*", re.IGNORECASE)
        labelname='firstLabel'
        for line in cfglines:
            if (line[0]!='#'):
                line=line.strip()
                if (defaultrc.match(line)):
                    (k,v)=line.split()
                    pxe_label=v
                elif (labelre.match(line)):
                    (k,v)=line.split()
                    labelname=v
                elif (kernelre.match(line)):
                    (k,v)=line.split()
                    kernel_list[labelname]=v
                elif (appendre.match(line)):
                    for entry in line[6:].split():
                        if (entry.find('initrd=') != -1):
                            initrd_list[labelname]=entry[7:]
        cfgfile.close()
        return kernel_list

    def getInstallImagesFromUrl(self, options, url, progress=lambda f:None, supports_32nonpae=True):

        # First of all: download the kernel and initrd defined by PXE. To behave nicely, we *must* try
        # to use them. In case that they are not XEN-enabled, we can try to use tha images from the source
        try:
            servaddr=getServAddr(options)
        except Exception, e:
            raise e
        os.chdir(vminstall.paths.imagedir)
        dlres={}
        success=1
        if options.pxe_label is None:
            log.debug('PXE: Error parsing pxelinux.cfg/default')
            raise PXEError(PXEError.E_NO_KERNEL)
        log.debug('k/i: %s, %s'% (kernel_list[options.pxe_label], initrd_list[options.pxe_label]))
        for i in [kernel_list[options.pxe_label], initrd_list[options.pxe_label]]:
            cmd="tftp "+servaddr+" -c get "+ i + " " + os.path.basename(i)+ "." + str(os.getpid())
            #cmd="tftp 147.2.207.153 -c get "+ i #for test 
            dlres[i]=commands.getstatusoutput(cmd)
            if not tftp_success (dlres[i]):
                success=0

        if success == 1:
            self.kfn = vminstall.paths.imagedir+os.path.basename(kernel_list[options.pxe_label])+"."+str(os.getpid())
            self.ifn = vminstall.paths.imagedir+os.path.basename(initrd_list[options.pxe_label])+"."+str(os.getpid())
            cmd = "kdumptool read_ikconfig '" + self.kfn + "' | grep '^CONFIG_X86_\(64_\)\?XEN=y'"
            if commands.getstatusoutput(cmd)[0] != 0:
                # The downloaded kernel is not xen-enabled
                log.debug("Kernel provided by PXE is not xen-enabled.")
                os.unlink(self.kfn)
                os.unlink(self.ifn)
                success=0

        if success == 1:
            log.debug("Using installation kernel/initrd provided by PXE: %s %s" % (self.kfn, self.ifn))
            return (self.kfn, self.ifn)
       
        # PXE did not provided xen-enabled kernel, try to get it from installation source (if any) (SUSE specific) 
        if url:
            log.debug("Trying to get kernel and initrd from installation source: %s" % (url))
            (self.kfn, self.ifn) = self.getInstallImagesFromPath(url, options.smboptions, progress, True)
            if self.kfn and self.ifn:
                return (self.kfn, self.ifn)

        log.debug("Unable to obtain xen-enabled kernel/initrd.")
        raise PXEError(PXEError.E_NO_KERNEL)

    def preprocessOptions(self, options):
        """Pulls settings out of extra_args."""
        if not options.extra_args:
            return
        args = options.extra_args.split()
        extra_args = ''
        for arg in args:
            if arg == 'TERM=xterm':
                # Discard.  Will be re-added later.
                pass
            elif arg.startswith('install='):
                if not options.source:
                    src = arg[8:]
                    if src.startswith('hd:/dev/'):
                        src = 'dev:/' + src[8:]
                    options.source = src
            elif arg.startswith('autoyast=') and not options.os_settings:
                options.os_settings = arg[9:]
            else:
                extra_args += arg + ' '
        options.extra_args = extra_args.strip()

    def setupOsSettings(self, guest, progress=lambda f:None):
        """Creates a temporary disk image, in which to pass the autoyast
        XML file."""
        ay = guest.options.os_settings
        if not ay or ay.startswith('ftp://') or ay.startswith('tftp://') or \
                     ay.startswith('http://') or ay.startswith('nfs://'):
            pass
        else:
            (fd, tmpdisk) = tempfile.mkstemp(prefix='autoyast.')
            ay_vdev=autoyast_vdev
            if guest.options.os_type != 'sles11' and not guest.options.full_virt:
                ay_vdev='xvdp'
            disk = vmdisks.Disk(pdev=tmpdisk, vdev=ay_vdev, ro=True, blocks=2048,
                                full_virt=guest.options.full_virt, vmname=guest.options.name)
            try:
                disk.setup(progress=lambda f:progress(f*0.5))
                util.mke2fsImage(tmpdisk, progress=lambda f:progress(0.5+f*0.25))
                mounted_disk = mount.TemporaryLoopMount(tmpdisk)
                mountpoint = mounted_disk.mount(options='loop')
                progress(0.8)
                try:
                    mode = os.stat(ay).st_mode
                    if stat.S_ISREG(mode):
                        shutil.copyfile(ay, mountpoint + '/autoinst.xml')
                    elif stat.S_ISDIR(mode):
                        shutil.copyfile(ay + '/autoinst.xml', mountpoint + '/autoinst.xml')
                        shutil.copyfile(ay + '/info', mountpoint + '/info')
                finally:
                    progress(0.9)
                    mounted_disk.unmount()
            except:
                os.unlink(tmpdisk)
                raise
            guest.tmpdisks.append(disk)
        progress(1.0)

    def getKernelList(self):
        return kernel_list

    def setPXENumber(self, pNumber):
        pxe_label = pNumber[:]
    
    def getInstallArgs(self, options):
        pxeargs=""
        os.chdir(vminstall.paths.imagedir)
        cfgfilename="default"
        if not os.path.exists(cfgfilename):
            return None
        if not options.pxe_label:
            return None
        labelre=re.compile('label '+options.pxe_label, re.IGNORECASE)
        startmatch = False
        log.debug('Label match to: label %s' % options.pxe_label)
        appendre=re.compile("append*", re.IGNORECASE)
        initrdre=re.compile("initrd*", re.IGNORECASE)
        cfgfile=open(cfgfilename)
        cfglines=cfgfile.readlines()
        for line in cfglines:
            if (line[0]!='#'):
                line=line.strip()
                if (labelre.match(line)):
                    startmatch = True
                if startmatch and (appendre.match(line)):
                    for entry in line[6:].split():
                        if (entry.find('initrd=') != -1):
                            pass
                        else:
                            pxeargs += entry+' '
                    break
        cfgfile.close()
        log.debug('Return label:%s   args: %s' %(options.pxe_label, pxeargs))
        return pxeargs
