# RedHat-specific operations
#
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import urlgrabber.grabber as grabber

from VMLinux import VMLinux
from exceptions import *
import shutil
import stat
import tempfile
import mount
import msg
import util
import os
import vmdisks

kickstart_vdev = vmdisks.get_possible_vdevs(False,False)[-1]

class VMRedHat(VMLinux):
    def _kernel(self, base, alternate=None):
        if alternate:
            path=alternate
        else:
            path="images/xen"
        return '%s/%s/vmlinuz' % (base,path,)
    def _initrd(self, base, alternate=None):
        if alternate:
            path=alternate
        else:
            path="images/xen"
        return '%s/%s/initrd.img' % (base,path,)

#    def _getHostAndPathFromNFS(self, url):
        # RedHat NFS URLs are of the form:
        # nfs:host:/path/to/install/source
        # SUSE's are of the form:
        # nfs://host/path/to/install/source

    def getInstallImagesFromUrl(self, options, url, progress=lambda f:None, supports_32nonpae=False):
        """Retrieves a Xen-enabled RedHat kernel and initrd from the URL, and
        copies them to temporary files.  Returns the (kernel, initrd) tuple of
        filenames."""
        kfd = ifd = None
        try:
            try:
                if url.startswith('http://') or url.startswith('ftp://'):
                    alt_path="images/pxeboot"
                    try:
                        kfd = grabber.urlopen(self._kernel(url))
                    except:
                        kfd = grabber.urlopen(self._kernel(url,alt_path))
                    try:
                        ifd = grabber.urlopen(self._initrd(url))
                    except:
                        ifd = grabber.urlopen(self._initrd(url,alt_path))
                elif url.startswith('nfs:'):
                    mnt = mount.TemporaryMount(url[4:])
                    mountpoint = mnt.mount(mnt)
                    kfd = open(self._kernel(mountpoint), 'r')
                    ifd = open(self._initrd(mountpoint), 'r')
                else:
                    raise InstSrcError(err=InstSrcError.E_UNSUPPORTED, details=url)
                kfn = util.copy_to_temp(kfd, prefix='vmlinuz.', progress=lambda f:progress(f*0.5))
                try:
                    ifn = util.copy_to_temp(ifd, prefix='initrd.', progress=lambda f:progress(0.5+f*0.5))
                except:
                    os.unlink(kfn)
                    raise
            except IOError, e:
                raise RuntimeError(msg.invalid_url + ': ' + str(e))
        finally:
            if kfd: kfd.close()
            if ifd: ifd.close()
        return (kfn, ifn)

    def getInstallImagesFromPath(self, path, progress=lambda f:None, supports_32nonpae=False):
        """Retrieves a Xen-enabled RedHat kernel and initrd from the base path, and
        copies them to temporary files.  Returns the (kernel, initrd) tuple of
        filenames."""
        kfd = ifd = None
        k_xen = i_xen = None
        if os.path.exists(self._kernel(path)):
            k_xen = self._kernel(path)
            i_xen = self._initrd(path)
        else:
            alt_path="images/pxeboot"
            k_xen = self._kernel(path,alt_path)
            i_xen = self._initrd(path,alt_path)
        try:
            kfd = open(k_xen, 'r')
            ifd = open(i_xen, 'r')
            kfn = util.copy_to_temp(kfd, prefix='vmlinuz.', progress=lambda f:progress(f*0.5))
            try:
                ifn = util.copy_to_temp(ifd, prefix='initrd.', progress=lambda f:progress(0.5+f*0.5))
            except:
                os.unlink(kfn)
                raise
        finally:
            if kfd: kfd.close()
            if ifd: ifd.close()
        return (kfn, ifn)

    def getInstallArgs(self, options, override=None):
        """Return RedHat's kernel arguments for installs."""
        if options.full_virt:
            return ''
        args = VMLinux.getInstallArgs(self, options)
        if type(options.source) is str:
            args += ' method=%s' % options.source
        if options.os_settings:
            if override is not None:
                args += ' ks=%s' % override
            else:
                args += ' ks=%s' % options.os_settings
        return args

    def preprocessOptions(self, options):
        """Pulls settings out of extra_args."""
        if options.os_type == 'rhel4' and options.full_virt:
            #options.usb = True
            options.usb_mouse_abs = False
        if not options.extra_args:
            return
        args = options.extra_args.split()
        extra_args = ''
        for arg in args:
            if arg.startswith('method='):
                if not options.source:
                    src = arg[7:]
                    if src.startswith('hd://'):
                        src = 'dev:/' + src[5:]
                    options.source = src
            elif arg.startswith('ks=') and not options.os_settings:
                options.os_settings = arg[3:]
            else:
                extra_args += arg + ' '
        options.extra_args = extra_args

    def setupOsSettings(self, guest, progress=lambda f:None):
        """Creates a temporary disk image, in which to pass the kickstart
        Text file."""
        ks = guest.options.os_settings
        if ks and ks.startswith('ks='):
            ks = ks.split('=',2)[1]
        if not ks or ks.startswith('ftp://') or ks.startswith('tftp://') or \
                     ks.startswith('http://') or ks.startswith('nfs://'):
            pass
        else:
            (fd, tmpdisk) = tempfile.mkstemp(prefix='kickstart.')
            ks_vdev = kickstart_vdev
            if guest.options.os_type != 'sles11' and not guest.options.full_virt:
                ks_vdev = 'xvdp'
            disk = vmdisks.Disk(pdev=tmpdisk, vdev=ks_vdev, ro=True, blocks=2048,
                                full_virt=guest.options.full_virt, vmname=guest.options.name)
            try:
                disk.setup(progress=lambda f:progress(f*0.5))
                util.mke2fsImage(tmpdisk, progress=lambda f:progress(0.5+f*0.25))
                mounted_disk = mount.TemporaryLoopMount(tmpdisk)
                mountpoint = mounted_disk.mount(options='loop')
                progress(0.8)
                try:
                    # Setup hd:xvdp:/<filename>
                    mode = os.stat(ks).st_mode
                    ks_file = None
                    if stat.S_ISREG(mode):
                        ks_file = os.path.basename(ks)
                        shutil.copyfile(ks, mountpoint + '/' + ks_file)
                    elif stat.S_ISDIR(mode):
                        if os.path.exists(ks + '/' + 'ks.cfg'):
                            ks_file = 'ks.cfg'
                            shutil.copyfile(ks + '/ks.cfg', mountpoint + '/' + 'ks.cfg')
                    if ks_file is not None:
                        override = "hd:xvdp:/" + ks_file
                        guest.options._install_args = self.getInstallArgs(guest.options, override)
                finally:
                    progress(0.9)
                    mounted_disk.unmount()
            except:
                os.unlink(tmpdisk)
                raise
            guest.tmpdisks.append(disk)
        progress(1.0)
