# Fully virtualized guest support
#
# Initially derived from virt-inst, Copyright 2005-2006 Red Hat, Inc.
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

from exceptions import *
from full_virt_guest import FullVirtGuest
from kvm_guest import KVMGuest
from vmdisks import Disk
from log import log
import vminstall.advanced
import caps

class FullVirtKVMGuest(FullVirtGuest, KVMGuest):
    def __init__(self, options):
        KVMGuest.__init__(self, options)

        self._cdrom = None
        self.features = {
            "acpi": True,
            "pae": True,  # required to install 64 bit!
        }
        if caps.is_kvm():
            self._qemu = '/usr/bin/qemu-kvm'
        else:
            self._qemu = '/usr/bin/qemu'

    def _get_features_xml(self):
        ret = []
        ret = vminstall.advanced.get_advanced_features_kvm(self.options)
        #if self.options.apic:
        #    ret.append('<apic/>')
        #for (k, v) in self.features.items():
        #    if v:
        #        ret.append('<%s/>' % k)
        return ret

    def GetConfigXml(self):
        if self.isInstall:
            if self.options.pxe_boot == True:
                bootdev = 'network'
            else:
                if type(self.options.source) is Disk:
                    if self.options.source.vdevtype == Disk.DEVICE_DISK:
                        bootdev = 'hd'
                    else:
                        bootdev = 'cdrom'
                else:
                    bootdev = 'cdrom'
        else:
            bootdev = 'hd'

        if caps.is_kvm():
            dtype = "<domain type='kvm'>"
        else:
            dtype = "<domain type='qemu'>"
        return [ dtype,
                   self.getCommonXml(),
                 [ '<os>',
                   [ "<type arch='%s' machine='pc'>hvm</type>" % caps.kvm_arch(),
                     "<boot dev='%s'/>" % bootdev 
                   ],
                   '</os>',
                   '<features>',
                     self._get_features_xml(),
                   '</features>',
                   self._get_localtime_xml(),
                   self._get_action_xml(),
                   '<devices>',
                   [ '<emulator>%s</emulator>' % self._qemu ],
                     self._get_disk_xml(),
                     self._get_network_xml(),
                     self._get_input_xml(),
                     self._get_graphics_xml(),
                   '</devices>' ],
                 '</domain>\n' ]

    def PostInstallCheck(self):
        """Verify that the first hard disk is bootable."""
        hds = filter(lambda disk: disk.vdevtype == Disk.DEVICE_DISK, self.options.disks)
        if hds:
            vdev = hds[0].block_attach()
            try:
                if hds[0].has_bootsector():
                    return
            finally:
                hds[0].block_detach()
        raise VmError(VmError.E_NO_BOOTLOADER)
