# Common code for all Xen guests
#
# Initially derived from virt-inst, Copyright 2005-2006 Red Hat, Inc.
# Copyright 2006-2010 Novell, Inc.
# Author: 
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

try:
    from xen.xend import XendOptions
except ImportError:
    pass
import caps
import os
import re
from log import log

# keytable_name_from_dom0:keymap_name_to_Xen
keytable = {
    "ar":"ar",
    "dk":"da",
    "de":"de",
    "de_ch":"de-ch",
    "de_ch-latin1":"de-ch",
    "de-latin1":"de",
    "de-latin1-nodeadkeys":"de",
    "gb":"en-gb",
    "us":"en-us",
    "es":"es",
    "":"et",
    "se_FI":"fi",
    "":"fo",
    "":"fr",
    "be":"fr-be",
    "":"fr-ca",
    "fr_CH":"fr_ch",
    "hr":"hr",
    "":"hu",
    "":"is",
    "it":"it",
    "jp106":"ja",
    "lt":"lt",
    "lv":"lv",
    "mk":"mk",
    "": "nl",
    "":"nl-be",
    "no":"no",
    "no-latin1":"no",
    "pl":"pl",
    "pt":"pt",
    "br":"pt-br",
    "br-abnt2":"pt-br",
    "ru":"ru",
    "sl":"sl",
    "":"sv",
    "th":"th",
    "tr":"tr",
    }

KEYBOARD_DIR = "/etc/sysconfig/keyboard"

def get_keymap(current_keymap=None):
    keymap = None
    if current_keymap != None:
        return current_keymap
    # TODO: Evaluate this functionality in light of kvm.
    # This functionality was removed so we could make some progress.
    if caps.is_xen():
        if XendOptions.instance().get_keymap():
            return None
    try:
        f = open(KEYBOARD_DIR, "r")
    except IOError, e:
        log.debug('Could not open "/etc/sysconfig/keyboard" ' + str(e))
    else:
        while 1:
            s = f.readline()
            if s == "":
                break
            s = re.sub("\s+", "", s)
            if re.search('^KEYTABLE=\".*\"$', s) != None:
                kt = s.split('"')[1]
                if ".map.gz" in kt:
                    kt = kt.rstrip(".map.gz")
                if keytable.has_key(kt):
                    keymap = keytable[kt]
        f.close
    return keymap

def read_keymaps():
    if caps.is_xen():
        location = "/usr/share/xen/qemu/keymaps"
    else:
        location = "/usr/share/qemu/keymaps"
    names = os.listdir(location)
    names.sort()
    keymaps = []
    for n in names:
        if n[0] != '.' and n != 'common' and n != 'modifiers':
            keymaps.append(n)
    return keymaps
