# Paths
#
# Copyright 2006-2007 Novell, Inc.
# Author: Charles Coffing <ccoffing@novell.com>
#
# This software may be freely redistributed under the terms of the GNU
# general public license.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import os

configdir = '/etc/xen/vm/'
gladedir = '/usr/share/vm-install/'
imagedir = '/var/lib/xen/images/'
jobdir = '/var/run/vm-install/'
logdir = '/var/log/xen/'

#-------------------------------------------------------------------------------
# Function: load_kvm_paths
# Parms:    none
# 
# This function is used to load the kvm-specific paths into our path variables.
# In addition, we create the paths as well since kvm will not create them on
# its own.
#
def load_kvm_paths():
    global configdir
    global imagedir
    global logdir
    global jobdir

    # Load the kvm-specific paths.
    configdir = '/etc/kvm/vm/'
    imagedir = '/var/lib/kvm/images/'
    logdir = '/var/log/kvm/'

    if os.geteuid() != 0:
        homedir = os.path.expanduser("~")
        configdir = homedir + '/kvm/vm/'
        imagedir = homedir + '/kvm/images/'
        logdir = homedir + '/kvm/log'

    # Create the kvm-specific paths.
    if not os.path.isdir(configdir):
        os.makedirs(configdir)
    if not os.path.isdir(imagedir):
        os.makedirs(imagedir)
    if not os.path.isdir(logdir):
        os.makedirs(logdir)
    if not os.path.isdir(jobdir):
        os.makedirs(jobdir)


#-------------------------------------------------------------------------------
# Function: load_xen_paths
# Parms:    none
# 
# This function is used to load the kvm-specific paths into our path variables.
# We assume that these paths are created if necessary elsewhere.
#
def load_xen_paths():
    global configdir
    global imagedir
    global logdir
    global jobdir

    # Load the xen-specific paths.
    configdir = '/etc/xen/vm/'
    imagedir = '/var/lib/xen/images/'
    logdir = '/var/log/xen/'

    if not os.path.isdir(jobdir):
        os.makedirs(jobdir)

#-------------------------------------------------------------------------------
# Function: load_qemu_paths
# Parms:    none
# 
# This function is used to load the qemu-specific paths into our path variables.
# In addition, we create the paths as well since qemu will not create them on
# its own.
#
def load_qemu_paths():
    global configdir
    global imagedir
    global logdir
    global jobdir

    # Load the qemu-specific paths.
    configdir = '/etc/qemu/vm/'
    imagedir = '/var/lib/qemu/images/'
    logdir = '/var/log/qemu/'

    if os.geteuid() != 0:
        homedir = os.path.expanduser("~")
        configdir = homedir + '/qemu/vm/'
        imagedir = homedir + '/qemu/images/'
        logdir = homedir + '/qemu/log'

    # Create the qemu-specific paths.
    if not os.path.isdir(configdir):
        os.makedirs(configdir)
    if not os.path.isdir(imagedir):
        os.makedirs(imagedir)
    if not os.path.isdir(logdir):
        os.makedirs(logdir)
    if not os.path.isdir(jobdir):
        os.makedirs(jobdir)

